/**************************************************************
*
*
*  ATMega8 to RFM02 connections:
*
*  PB2 (SS)   -> nSEL=CS
*  PB3 (MOSI) -> SDI
*  PB5 (SCK)  -> SCK
*
*  PB0        <- nIRQ
*  PB1        -> FSK
*
*  
*
*/

#include <avr/io.h>
#include <util/delay.h>

#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit)) 
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))  

#define F_CPU 4000000UL



void SendCommand(unsigned int data)
{
    cbi(PORTB,PB2); /* Chip Select active low */

	_delay_us(1.0); /* Wait tss >= 10 ns */
	
	if (data > 0xff)
	{
		SPDR=((data & 0xff00)>>8);
		while(! (SPSR & (1<<SPIF) ) ); /* Wait until data shifted out (SPIF bit set) */
	}

	SPDR=(data & 0x00ff);
	while(! (SPSR & (1<<SPIF) ) ); /* Wait until data shifted out (SPIF bit set) */

	_delay_us(1.0); /* Wait tsh >= 10 ns */

    sbi(PORTB,PB2); /* Chip Select high */

	_delay_us(1.0); /* Wait tshi >= 25 ns (time until next CS=low) */
}

void shift_out_txdata(unsigned char data)
{
	/* MSB first */
	unsigned char i;
	for (i=0;i<8;i++) /* Do 8 bits */
	{
		while( (PINB & (1<<PB0)) ); /* Wait as long as nIRQ is high */
		
		while( !(PINB & (1<<PB0)) ); /* Wait as long as nIRQ is low */

		/* nIRQ is going high right now, set FSK pin */
		
		if (data & 0x80) /* look at MSB */
			sbi(PORTB,PB1);
		else
			cbi(PORTB,PB1);
		
		data = data << 1; /* Shift bits 1 place to left */
	}			
}

void tx(unsigned char* data, unsigned char datalength)
{
	unsigned char i;
	
	SendCommand(0xC038); /* Go on air */

	_delay_ms(10.0); /* Wait tsp + tsx = 250us + 5 ms */

	shift_out_txdata(0xAA); /* preamble */
	shift_out_txdata(0xAA);
	shift_out_txdata(0xAA);
	shift_out_txdata(0xAA);
	shift_out_txdata(0xAA);
	shift_out_txdata(0x2D); /* sync-pattern = 0x2DD4 */
	shift_out_txdata(0xD4);
	for (i=0;i<datalength;i++) /* Transmit data */
	{
		shift_out_txdata(data[i]);
	}

	while( (PINB & (1<<PB0)) ); /* Wait as long as nIRQ is high */
	while( !(PINB & (1<<PB0)) ); /* Wait as long as nIRQ is low */

	SendCommand(0xC000); /* Off air  */
} 


int main(void)
{
	unsigned char x;
	unsigned char txdata[] = {"RFM02 ISM 434MHz FSK TEST TEST !"};

	PORTB=(1<<PB2); /* CS = high */

	DDRB=(1<<PB5 | 1<<PB3 | 1<<PB2 | 1<<PB1 ); /* 1=output 0=input */

	/* Setup hardware-SPI */
	SPCR=(1<<SPE |		/* set bit6 = SPI Enable */
	      1<<MSTR |     /* set bit4 = SPI-Master */
		  1<<SPR0);     /* SPR1=0 SPR0=1 -> Fsck = Fosc/16 = 250 kHz */

	/* Give RFM02 some startup time */
	for (x=0;x<25;x++) _delay_ms(10);

	/* Init RFM02 */
	SendCommand(0x8880); /* Config Set Command: band=433 clk=1MHz (don't use it anyway) cap=8 (mid) FSK-shift=30kHz */
	SendCommand(0xA708); /* Freq 434.500 MHz */
	SendCommand(0xC88F); /* Data Rate = 2395 bps = 2k4 */
	SendCommand(0xD240); /* Set PLL command */
	SendCommand(0xC2A0); /* Low Bat Detect & TX Bit Sync Command: dwc=1 ebs=1 T=0 */
	SendCommand(0xB4);   /* RF output power: -12dB relative to maximum */

	_delay_ms(50.0);

	while(1)	/* loop forever */
	{
		/* transmit packet */
		tx(txdata,32);

		/* delay 2 seconds */
		for (x=0;x<200;x++) _delay_ms(10.0);

   }

	return 0;
}
